/*
     This file is part of GNUnet

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/

/**
 * This module is responsible to lookup queries.
 * @file include/lookup.h
 * @author Christian Grothoff
 **/

#ifndef LOOKUP_H
#define LOOKUP_H

#include "config.h"
#include "gnettypes.h"
#include "util/cron.h"
#include "util/storage.h"
#include "configuration.h"
#include "symcipher.h"


typedef struct CoLevel_ {
  /* handle to the collision file for this level */
  int handle;
  /* size of this collision table (mask) */
  int mask;
  /* offset into the hashcode to mask out (0=.a, 4=.b, 8=.c, etc). */
  int offset;
  /* mutex for access to the file */
  Mutex lock;
  /* the next collision level */
  struct CoLevel_ * next;
} CollisionLevel;


/**
 * Initialize the lookup module.
 **/
void initLookup();

/**
 * Initialize the lookup module (full init)
 **/
void fullInitLookup();

/**
 * Shutdown the lookup module.
 **/
void doneLookup();

/**
 * Is space available on the drive? Returns the number of
 * blocks (1k) available. If there would be no space except
 * if less important content is discarded, this method 
 * *deletes* 1k of the less important content and returns 
 * 1. If there is only more important content, the
 * method returns 0.
 * @param importance how important is the content?
 **/
int spaceAvailable(int importance);

/**
 * Lookup the file on the local machine. If found,
 * send it back.
 * @param query the triple-hash of the content
 * @param qp the polciy (for us: priority)
 * @param callback procedure to call on the results,
 *        must take three arguments: NULL, a pointer
 *        to the CONTENT_Block, and the query.
 * @returns YES if found, NO if not.
 **/
int tripleHashLookup(HashCode160 * query,
		     QUERY_POLICY qp,
		     int (* callback)(HostIdentity*,CONTENT_Block*,
				       HashCode160*));

/**
 * Add an entry to the cache. The ->left and ->right
 * references are 0 when this method is called (new entry!)
 * @return OK on success, SYSERR on error
 **/
int addEntry(ContentIndex * entry);

/**
 * Obtain the number of shared files.
 **/
unsigned int getSharedFileCount();

/**
 * Add a name to the list of filenames.
 **/
int appendFilename(char * filename);

/**
 * Invoke a method on each of the filenames of
 * the indexed files. If the method returns
 * SYSERR, remove the file from the list of
 * indexed files!
 * @return the number of shared files (after changes
 *         caused by this call)
 **/
int forEachIndexedFile(int (*method)(FileName,int));

/**
 * Lookup the given Hash in the Hashtable-cache structure
 * (load blocks as needed).
 * @returns NO if not found, YES if found
 **/
int findEntry(HashCode160 * tripleHash,
	      ContentIndex * result);
 

/**
 * Call a method for each entry in the lookup table.
 * VERY expensive (will take quite a while to run),
 * so don't do this unless under exceptional circumstances.
 * @param method the method to call, abort early if 
 *        method returns SYSERR
 * @return the number of entries in the database
 **/
int forEachLookupEntry(int (*method)(ContentIndex * entry,
				     CollisionLevel * level,
				     size_t pos));

#define LOOKUP_LIPS_COUNT 32
#define LOOKUP_FACTOR 128
#define LOOKUP_LIPSPART_COUNT ((CACHE_HASH_TABLE_SIZE/LOOKUP_FACTOR))

#endif
/* end of lookup.h */
